@@ -114,6 +114,7 @@ void thdata::reset_data()
   this->di_length = false;
   this->di_bearing = false;
   this->di_gradient = false;
+  this->di_backlength = false;
   this->di_backbearing = false;
   this->di_backgradient = false;
   this->di_depth = false;
@@ -379,6 +380,7 @@ void thdata::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
             case TT_DATALEG_LENGTH:
             case TT_DATALEG_BEARING:
             case TT_DATALEG_GRADIENT:
+            case TT_DATALEG_BACKLENGTH:
             case TT_DATALEG_BACKBEARING:
             case TT_DATALEG_BACKGRADIENT:
             case TT_DATALEG_NOTES:
@@ -573,6 +575,7 @@ void thdata::set_data_calibration(int nargs, char ** args)
     to_set = true;
     switch (itid) {
       case TT_DATALEG_LENGTH:
+      case TT_DATALEG_BACKLENGTH:
       case TT_DATALEG_COUNT:
       case TT_DATALEG_DEPTH:
       case TT_DATALEG_X:
@@ -881,6 +884,7 @@ void thdata::set_data_instrument(int nargs, char ** args)
       case TT_DATALEG_LENGTH:
       case TT_DATALEG_BEARING:
       case TT_DATALEG_GRADIENT:
+      case TT_DATALEG_BACKLENGTH:
       case TT_DATALEG_BACKBEARING:
       case TT_DATALEG_BACKGRADIENT:
       case TT_DATALEG_DEPTH:
@@ -1117,7 +1121,30 @@ void thdata::set_data_data(int nargs, char ** args)
           break;
         this->di_length = true;
         break;
+
+        case TT_DATALEG_BACKLENGTH:
+          if (this->di_backlength || this->di_fromcount || this->di_tocount) {
+            err_duplicate = true;
+            break;
+          }
+          if (this->di_count) {
+            err_inimm = true;
+            break;
+          }
+          switch (this->d_type) {
+            case TT_DATATYPE_NORMAL:
+            case TT_DATATYPE_DIVING:
+            case TT_DATATYPE_CYLPOLAR:
+              break;
+            default:
+              err_itype = true;
+          }
+          if (err_itype)
+            break;
+          this->di_backlength = true;
+          break;
         
+
       case TT_DATALEG_BEARING:
         if (this->di_bearing) {
           err_duplicate = true;
@@ -1490,7 +1517,7 @@ void thdata::set_data_data(int nargs, char ** args)
     case TT_DATATYPE_NORMAL:
       all_data = (this->di_count ||
         (this->di_fromcount && this->di_tocount) ||
-        this->di_length) && 
+        this->di_length || this->di_backlength) &&
         (this->di_bearing || this->di_backbearing) && 
         (this->di_gradient || this->di_backgradient);
       break;
@@ -1498,7 +1525,7 @@ void thdata::set_data_data(int nargs, char ** args)
     case TT_DATATYPE_DIVING:
       all_data = (this->di_count ||
         (this->di_fromcount && this->di_tocount) ||
-        this->di_length) && (this->di_bearing || this->di_backbearing) 
+        this->di_length || this->di_backlength) && (this->di_bearing || this->di_backbearing)
         && (this->di_depth ||
         (this->di_fromdepth && this->di_todepth) || this->di_depthchange);
       break;
@@ -1524,6 +1551,7 @@ void thdata::set_data_data(int nargs, char ** args)
         case TT_DATALEG_TO:
         case TT_DATALEG_DIRECTION:
         case TT_DATALEG_LENGTH:
+        case TT_DATALEG_BACKLENGTH:
         case TT_DATALEG_BEARING:
         case TT_DATALEG_BACKBEARING:
         case TT_DATALEG_GRADIENT:
@@ -1755,6 +1783,26 @@ void thdata::insert_data_leg(int nargs, char ** args)
             ththrow("invalid length reading -- {}", args[carg]);
         }
         break;
+
+        case TT_DATALEG_BACKLENGTH:
+          thparse_double(val_id, val, args[carg]);
+          switch (val_id) {
+            case TT_SV_NUMBER:
+              val = this->dlc_length.evaluate(val);
+              val = this->dlu_length.transform(val);
+              if (val < 0.0)
+                ththrow("negative length reading -- {}", args[carg]);
+              else
+                this->cd_leg->backlength = val;
+              break;
+  //          case TT_SV_NAN:
+  //            this->cd_leg->length = val;
+  //            break;
+            default:
+              ththrow("invalid backwards length reading -- {}", args[carg]);
+          }
+          break;
+
         
       case TT_DATALEG_BEARING:
         if (this->dlu_bearing.get_units() == TT_TFU_DEG)
@@ -2757,6 +2805,7 @@ void thdata::start_group() {
   tmp->di_length = this->cgroup->di_length; 
   tmp->di_bearing = this->cgroup->di_bearing; 
   tmp->di_gradient = this->cgroup->di_gradient;
+  tmp->di_backlength = this->cgroup->di_backlength;
   tmp->di_backbearing = this->cgroup->di_backbearing; 
   tmp->di_backgradient = this->cgroup->di_backgradient;
   tmp->di_depth = this->cgroup->di_depth; 
