@@ -26,10 +26,12 @@
  * --------------------------------------------------------------------
  */
  
+#include <time.h>
+
 #ifndef thdate_h
 #define thdate_h
 
-#define thdate__bufflen 255
+#define thdate_bufflen 255
 
 /**
  * Convert date into decimal year.
@@ -49,6 +51,8 @@ void thdate_y2d(double dy, int  & year, int & month, int & day, int & hour, int
  */
 enum {
   TT_DATE_FMT_ISO,
+  TT_DATE_FMT_LOCALE,
+  TT_DATE_FMT_LOCALE_LONG,
   TT_DATE_FMT_Y,
   TT_DATE_FMT_UTF8_ISO,
   TT_DATE_FMT_UTF8_Y,
@@ -67,7 +71,7 @@ class thdate {
 
   int syear,  ///< Start date year
     smonth,  ///< Start date month
-    sday,  ///< Stard date day
+    sday,  ///< Start date day
     shour,  ///< Star date hour
     smin,  ///< Star date minute
     eyear,  ///< End date year
@@ -79,7 +83,7 @@ class thdate {
   double ssec, ///< Start date seconds
     esec;  ///< End date seconds
     
-  static char dstr[thdate__bufflen]; ///< String for given date.
+  char dstr[thdate_bufflen]; ///< String for given date.
 
   friend bool operator < (const thdate &, const thdate &);  ///< Less operator.
   friend bool operator <= (const thdate &, const thdate &);  ///< Less operator.
@@ -90,7 +94,7 @@ class thdate {
 
 
   /**
-   * Compare two dates if first is less than sencond, item by item.
+   * Compare two dates if first is less than second, item by item.
    */
    
   static bool is_less(int y1, int m1, int d1, int h1, int n1, double s1,
@@ -98,7 +102,7 @@ class thdate {
 
 
   /**
-   * Compare two dates if first is greater than sencond, item by item.
+   * Compare two dates if first is greater than second, item by item.
    */
    
   static bool is_greater(int y1, int m1, int d1, int h1, int n1, double s1,
@@ -117,6 +121,7 @@ class thdate {
    
   void reset();
 
+  void reset_current();
     
   /**
    * Standard constructor.
@@ -136,7 +141,7 @@ class thdate {
    * Parse string to date.
    */
    
-  void parse(char * src);
+  void parse(const char * src);
   
   
   /**
@@ -179,6 +184,13 @@ class thdate {
    */
    
   double get_end_year();
+
+
+  /**
+   * Return end average as double value.
+   */
+
+  double get_average_year();
   
   
   /**
@@ -189,6 +201,17 @@ class thdate {
 
   void set_file_date(char * fname);
   
+  /**
+   * Get the start or end date as time_t format
+   */
+  time_t get_start_t_time();
+  time_t get_end_t_time();
+  
+  /**
+   * Get the start or end date as number of days since 1 Jan 1900 for Survex export
+   */
+  int get_start_days1900();
+  int get_end_days1900();
 };
 
 
