@@ -33,7 +33,7 @@
 
 #define THMAXFREC 64
 
-const long thinput::max_line_size = 8128;
+const long thinput::max_line_size = 65535;
 
 
 enum {TT_UNKNOWN_INPUT, TT_INPUT, TT_ENCODING};
@@ -51,18 +51,8 @@ thinput::ifile::ifile(ifile * fp)
   this->lnumber = 0;
   this->encoding = TT_UTF_8;
   this->prev_ptr = fp;
-  this->next_ptr = NULL;
 }
 
-
-thinput::ifile::~ifile()
-{
-  this->close();
-  if (this->next_ptr != NULL)
-    delete this->next_ptr;
-}
-
-
 void thinput::ifile::close()
 {
   if (this->sh.is_open()) {
@@ -72,8 +62,13 @@ void thinput::ifile::close()
   }
 }
 
-
-bool thinput::ifile::is_equal(struct stat * buf)
+bool thinput::ifile::is_equal(
+#ifdef THMSVC
+struct _stat
+#else
+struct stat
+#endif
+* buf)
 {
   if ((this->st.st_ino == buf->st_ino) && \
       (this->st.st_dev == buf->st_dev))
@@ -86,23 +81,17 @@ bool thinput::ifile::is_equal(struct stat * buf)
 thinput::thinput()
 {
   this->cmd_sensitivity = true;
+  this->input_sensitivity = true;
   this->scan_search_path = false;
-  this->first_ptr = new ifile(NULL);
-  this->last_ptr = this->first_ptr;
-  this->lnbuffer = new char [thinput::max_line_size];
+  this->first_ptr = std::unique_ptr<ifile>(new ifile(nullptr));
+  this->last_ptr = this->first_ptr.get();
+  this->lnbuffer = std::unique_ptr<char[]>(new char [thinput::max_line_size]);
   this->pifo = false;
   this->pifoid = NULL;
   this->pifoproc = NULL;
+  this->report_missing = false;
 }
 
-
-thinput::~thinput()
-{
-  delete this->first_ptr;
-  delete this->lnbuffer;
-}
-
-
 void thinput::set_cmd_sensitivity(bool cs)
 {
   this->cmd_sensitivity = cs;
@@ -157,7 +146,7 @@ bool thinput::get_sp_scan()
 }
 
 
-void thinput::set_file_name(char * fname)
+void thinput::set_file_name(const char * fname)
 {
   this->file_name = fname;
 }
@@ -169,7 +158,7 @@ char * thinput::get_file_name()
 }
 
 
-void thinput::set_search_path(char * spath)
+void thinput::set_search_path(const char * spath)
 {
   this->search_path.clear();
 #ifdef THWIN32
@@ -179,7 +168,7 @@ void thinput::set_search_path(char * spath)
 #endif
 }
 
-void thinput::set_file_suffix(char * fsx)
+void thinput::set_file_suffix(const char * fsx)
 {
   this->file_suffix.clear();
   thsplit_strings(& this->file_suffix, fsx, ':'); 
@@ -196,10 +185,10 @@ void thinput::open_file(char * fname)
   ifile * ifptr;
   ifptr = this->last_ptr;
   while (ifptr->sh.is_open() && (ifptr->next_ptr != NULL))
-    ifptr = ifptr->next_ptr;
+    ifptr = ifptr->next_ptr.get();
   if (ifptr->sh.is_open() && (ifptr->next_ptr == NULL)) {
-    ifptr->next_ptr = new ifile(ifptr);
-    ifptr = ifptr->next_ptr;
+    ifptr->next_ptr = std::unique_ptr<ifile>(new ifile(ifptr));
+    ifptr = ifptr->next_ptr.get();
   }
   
   // now, let's open the file
@@ -265,13 +254,17 @@ void thinput::open_file(char * fname)
   unsigned int n_rec = 0;
   
   if (!ifptr->sh.is_open()) {
-    if (srcfile != NULL) {
-      ththrow(("can't open file for input -- %s", \
-        fname));
+    if ((srcfile != NULL) || (this->report_missing)) {
+      ththrow("can't open file for input -- {}", fname);
     }
   }
   else {
-    stat(ifptr->name.get_buffer(), &ifptr->st);
+#ifdef THMSVC
+    _stat
+#else
+    stat
+#endif
+    (ifptr->name.get_buffer(), &ifptr->st);
     tmptr = ifptr->prev_ptr;
 #ifdef THWIN32
     while ((tmptr != NULL) && (n_rec < THMAXFREC)) {
@@ -320,22 +313,25 @@ void thinput::close_file()
     thprintf("close file -- %s\n", this->last_ptr->name.get_buffer());
 #endif
     this->last_ptr->close();
-    if (this->last_ptr->prev_ptr != NULL)
+    if (this->last_ptr->prev_ptr != NULL) {
       this->last_ptr = this->last_ptr->prev_ptr; 
+      // Is next cmd it really like this ?????
+      this->last_ptr->next_ptr = NULL;
+    }
   }
 }
 
 void thinput::reset()
 {
   // first, let's close all open files
-  ifile * iptr = this->first_ptr;
+  ifile * iptr = this->first_ptr.get();
   while(iptr != NULL) {
     iptr->close();
-    iptr = iptr->next_ptr;
+    iptr = iptr->next_ptr.get();
   }
   
   // set pointer to the first ifile
-  this->last_ptr = this->first_ptr;
+  this->last_ptr = this->first_ptr.get();
   this->open_file(this->file_name);  
 }
 
@@ -362,7 +358,7 @@ char * thinput::read_line()
     }
 
     // OK, we can read the line
-    this->last_ptr->sh.getline(this->lnbuffer, thinput::max_line_size);
+    this->last_ptr->sh.getline(this->lnbuffer.get(), thinput::max_line_size);
     this->last_ptr->lnumber++;
 
     // Check, if reading was OK.
@@ -373,8 +369,8 @@ char * thinput::read_line()
     }
     
     // now let's find last non white character
-    lnlen = strlen(this->lnbuffer);
-    idxptr = this->lnbuffer + lnlen - 1;
+    lnlen = (long)strlen(this->lnbuffer.get());
+    idxptr = this->lnbuffer.get() + lnlen - 1;
     while ((lnlen > 0) && (*idxptr < 33)) {
       idxptr--;
       lnlen--;
@@ -387,19 +383,19 @@ char * thinput::read_line()
     // join backslash ended lines together
     if (*idxptr == '\\') {
       if (mline) {
-        this->linebf.strncat(this->lnbuffer, lnlen - 1);
+        this->linebf.strncat(this->lnbuffer.get(), lnlen - 1);
       }
       else {
-        this->linebf.strncpy(this->lnbuffer, lnlen - 1);
+        this->linebf.strncpy(this->lnbuffer.get(), lnlen - 1);
         mline = true;
       }
       continue;
     }
     else {
       if (mline)
-        this->linebf.strncat(this->lnbuffer, lnlen);
+        this->linebf.strncat(this->lnbuffer.get(), lnlen);
       else
-        this->linebf.strncpy(this->lnbuffer, lnlen);
+        this->linebf.strncpy(this->lnbuffer.get(), lnlen);
     }
     
     mline = false;
@@ -418,12 +414,14 @@ char * thinput::read_line()
       switch (thmatch_token(this->cmdbf.get_buffer(), thtt_input)) {
       
         case TT_INPUT:
-          if (this->tmpmb.get_size() != 1)
-            therror(("%s [%d] -- one input file name expected -- %s", \
-              this->get_cif_name(), this->get_cif_line_number(), \
-              this->valuebf.get_buffer()))
-          else
-            this->open_file(*(this->tmpmb.get_buffer()));
+          if (this->input_sensitivity) {
+            if (this->tmpmb.get_size() != 1)
+              therror(("%s [%d] -- one input file name expected -- %s", \
+                this->get_cif_name(), this->get_cif_line_number(), \
+                this->valuebf.get_buffer()))
+            else
+              this->open_file(*(this->tmpmb.get_buffer()));
+          }
           continue;
           
         case TT_ENCODING:
@@ -447,7 +445,7 @@ char * thinput::read_line()
     // otherwise check if comment
     else {
       idxptr = this->linebf.get_buffer();
-      lnlen = strlen(idxptr);
+      lnlen = (long)strlen(idxptr);
       while ((lnlen > 0) && (*idxptr < 33)) {
         lnlen--;
         idxptr++;
@@ -488,9 +486,51 @@ char * thinput::get_cif_name()
 {
   return this->last_ptr->name.get_buffer();
 }
-  
 
-unsigned int thinput::get_cif_line_number()
+
+
+char * thinput::get_cif_path()
+{
+  static thbuffer cifpath;
+  thsplit_fpath(&cifpath, this->last_ptr->name.get_buffer());
+  return cifpath.get_buffer();
+}
+
+const char * thinput::get_cif_abspath(const char * fname)
+{
+  static thbuffer pict_path;
+  pict_path.guarantee(1024);
+  thassert(getcwd(pict_path.get_buffer(),1024) != NULL);  
+  long i;
+  pict_path += "/";  
+  if (thpath_is_absolute(this->last_ptr->name.get_buffer()))
+	  pict_path = this->last_ptr->name.get_buffer();
+  else
+	  pict_path += this->last_ptr->name.get_buffer();
+  char * pp = pict_path.get_buffer();
+  for(i = (long)strlen(pp); i >= 0; i--) {
+	if ((pp[i] == '/') || (pp[i] == '\\')) {
+	  break;
+	} else
+	  pp[i] = 0;
+  }
+  if (strlen(pp) == 0)
+	pict_path = "/";
+  if ((fname != NULL) && (strlen(fname) > 0)) {
+	  if (thpath_is_absolute(fname))
+		  pict_path = fname;
+	  else
+		  pict_path += fname;
+  }
+  pp = pict_path.get_buffer();
+  for(i = (long)strlen(pp); i >= 0; i--) {
+	if (pp[i] == '\\')
+	  pp[i] = '/';
+  }
+  return pict_path.get_buffer();
+}
+
+unsigned long thinput::get_cif_line_number()
 {
   return this->last_ptr->lnumber;
 }
@@ -509,4 +549,19 @@ void thinput::print_if_opened(void (* pifop)(char *), bool * printed) {
 }
 
 
+void thinput::set_input_sensitivity(bool s)
+{
+  this->input_sensitivity = s;
+}
+
+
+bool thinput::get_input_sensitivity()
+{
+  return this->input_sensitivity;
+}
+
+bool thinput::is_first_file()
+{
+  return ((this->first_ptr == NULL) || (this->first_ptr->next_ptr == NULL));
+}
 
