@@ -30,6 +30,7 @@
 #define thlayoutln_h
 
 #include <list>
+#include "thlayoutclr.h"
 
 /**
  * Layout line class.
@@ -43,10 +44,14 @@ class thlayoutln {
  
   thlayoutln * next_line;
     
-  char * line;
+  const char * line;
+
+  thlayout_color sclr; ///< Symbol color.
   
   int smid;  ///< Symbol macro ID
-
+  
+  const char * path;  ///< Source file.
+  
   thlayoutln(); ///< Default constructor
 };
 
