@@ -39,6 +39,8 @@
 #include "thscrapis.h"
 #include "thsurvey.h"
 #include "thsymbolset.h"
+#include "thsketch.h"
+#include "thcsdata.h"
 
 #define EXPORT3D_INVISIBLE true
 
@@ -55,10 +57,11 @@ thscrap::thscrap()
   this->proj_prev_scrap = NULL;
   this->xscrap = NULL;
   
-  this->R = 1.0;
-  this->G = 1.0;
-  this->B = 1.0;
+  this->RGBsrc = 0;
   
+  this->z = thnan;
+  this->a = thnan;
+
   this->flip = TT_SCRAP_FLIP_NONE;
   
   this->scale = 1.0;
@@ -72,6 +75,7 @@ thscrap::thscrap()
   this->scale_p2y = 0.0;
   this->scale_p9 = false;
   this->reset_limits();
+  this->scale_cs = TTCS_LOCAL;
 
   this->reset_transformation();
 
@@ -122,34 +126,38 @@ int thscrap::get_cmd_nargs()
 }
 
 
-char * thscrap::get_cmd_end()
+const char * thscrap::get_cmd_end()
 {
   // insert endcommand if multiline command
   return NULL;
 }
 
 
-char * thscrap::get_cmd_name()
+const char * thscrap::get_cmd_name()
 {
   // insert command name here
   return "scrap";
 }
 
 
-thcmd_option_desc thscrap::get_cmd_option_desc(char * opts)
+thcmd_option_desc thscrap::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_scrap_opt);
   if (id == TT_SCRAP_UNKNOWN)
     return thdataobject::get_cmd_option_desc(opts);
-  else
-    return thcmd_option_desc(id);
+  else switch (id) {
+    case TT_SCRAP_SKETCH:
+      return thcmd_option_desc(id,3);
+    default:
+      return thcmd_option_desc(id);
+  }
 }
 
 
 static const thstok thtt_scrap_flips[] = {
-  {"none", TT_SCRAP_FLIP_NONE},
   {"horiz", TT_SCRAP_FLIP_HORIZ},
   {"horizontal", TT_SCRAP_FLIP_HORIZ},
+  {"none", TT_SCRAP_FLIP_NONE},
   {"vert", TT_SCRAP_FLIP_VERT},
   {"vertical", TT_SCRAP_FLIP_VERT},
   {NULL, TT_SCRAP_FLIP_UNKNOWN},
@@ -161,6 +169,7 @@ void thscrap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
 {
   
   thdb2dprjpr projection;
+
   if (cod.id == 1)
     cod.id = TT_DATAOBJECT_NAME;
     
@@ -170,13 +179,13 @@ void thscrap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
 //      if (th_is_keyword(*args))
 //        this->name = this->db->strstore(*args);
 //      else 
-//        ththrow(("invalid keyword -- %s", *args));
+//        ththrow("invalid keyword -- %s", *args);
 //      break;
       
     case TT_SCRAP_PROJECTION:
       projection = this->db->db2d.parse_projection(*args);
       if (!projection.parok)
-        ththrow(("invalid parameters of projection"));
+        ththrow("invalid parameters of projection");
       this->proj = projection.prj;
       break;
     
@@ -191,13 +200,17 @@ void thscrap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
     case TT_SCRAP_3D:
       this->d3 = thmatch_token(*args, thtt_onoffauto);
       if (this->d3 == TT_UNKNOWN_BOOL)
-        ththrow(("invalid -3d switch -- %s", *args));
+        ththrow("invalid -3d switch -- {}", *args);
       break;
     
     case TT_SCRAP_FLIP:
       this->flip = thmatch_token(*args, thtt_scrap_flips);
       if (this->flip == TT_SCRAP_FLIP_UNKNOWN)
-        ththrow(("invalid -flip switch -- %s", *args));
+        ththrow("invalid -flip switch -- {}", *args);
+      break;
+
+    case TT_SCRAP_SKETCH:
+      this->parse_sketch(args, argenc);
       break;
     
     // if not found, try to set fathers properties  
@@ -206,12 +219,6 @@ void thscrap::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
   }
 }
 
-void thscrap::self_delete()
-{
-  delete this;
-}
-
-
 void thscrap::self_print_properties(FILE * outf)
 {
   thdataobject::self_print_properties(outf);
@@ -219,6 +226,7 @@ void thscrap::self_print_properties(FILE * outf)
   fprintf(outf,"\tprojection: %d\n",this->proj->id);
   fprintf(outf,"\tscale: %f\n",this->scale);
   fprintf(outf,"\tz: %f\n",this->z);
+  fprintf(outf,"\ta: %f\n",this->a);
   // insert intended print of object properties here
 }
 
@@ -229,8 +237,8 @@ void thscrap::parse_scale(char * ss)
   int npar = this->db->db2d.mbf.get_size();
   char ** pars = this->db->db2d.mbf.get_buffer();
   if ((npar < 1) || ((npar > 3) && (npar != 8) && (npar != 9)))
-    ththrow(("invalid number of scale arguments -- %d",npar))
-  double n1, n2;
+    ththrow("invalid number of scale arguments -- {}",npar);
+  double n1 = 1.0, n2 = 1.0;
   bool p9 = false;
   int sv, ux = 0, n2x = 0;
   thtflength lentf;
@@ -253,56 +261,58 @@ void thscrap::parse_scale(char * ss)
 #define parse_scalep9(var,parn) \
     thparse_double(sv,var,pars[parn]); \
     if (sv != TT_SV_NUMBER) \
-      ththrow(("real number required -- %s", pars[parn]));
+      ththrow("real number required -- {}", pars[parn]);
+  
+  if (ux > 0) {
+    lentf.parse_units(pars[ux]);
+  }
 
   if (p9) {
 //    thparse_double(sv,this->scale_r1x,pars[0]);
 //    if ((sv != TT_SV_NUMBER) || (n1 <= 0.0))
-//      ththrow(("real positive number required -- %s", pars[0]));
-      parse_scalep9(this->scale_p1x,0)
-      parse_scalep9(this->scale_p1y,1)
-      parse_scalep9(this->scale_p2x,2)
-      parse_scalep9(this->scale_p2y,3)
+//      ththrow("real positive number required -- %s", pars[0]);
+    parse_scalep9(this->scale_p1x,0)
+    parse_scalep9(this->scale_p1y,1)
+    parse_scalep9(this->scale_p2x,2)
+    parse_scalep9(this->scale_p2y,3)
+    if (this->cs == TTCS_LOCAL) {
       parse_scalep9(this->scale_r1x,4)
       parse_scalep9(this->scale_r1y,5)
       parse_scalep9(this->scale_r2x,6)
       parse_scalep9(this->scale_r2y,7)
-      if (hypot(this->scale_r1x - this->scale_r2x,
-        this->scale_r1y - this->scale_r2y) == 0.0)
-        ththrow(("zero scale real length"));
-      if (hypot(this->scale_p1x - this->scale_p2x,
-        this->scale_p1y - this->scale_p2y) == 0.0)
-        ththrow(("zero scale picture length"));        
-  }
-  else {
+      if (ux > 0) {
+        this->scale_r1x = lentf.transform(this->scale_r1x);
+        this->scale_r1y = lentf.transform(this->scale_r1y);
+        this->scale_r2x = lentf.transform(this->scale_r2x);
+        this->scale_r2y = lentf.transform(this->scale_r2y);
+      }
+    } else {
+      this->scale_cs = this->cs;
+      this->read_cs(pars[4], pars[5], this->scale_r1x, this->scale_r1y);
+      this->read_cs(pars[6], pars[7], this->scale_r2x, this->scale_r2y);
+    }
+    if (std::hypot(this->scale_r1x - this->scale_r2x,
+      this->scale_r1y - this->scale_r2y) == 0.0)
+      ththrow("zero scale real length");
+    if (std::hypot(this->scale_p1x - this->scale_p2x,
+      this->scale_p1y - this->scale_p2y) == 0.0)
+      ththrow("zero scale picture length");        
+  } else {
     // let's parse first number
     thparse_double(sv,n1,pars[0]);
     if ((sv != TT_SV_NUMBER) || (n1 <= 0.0))
-      ththrow(("real positive number required -- %s", pars[0]));
-      
+      ththrow("real positive number required -- {}", pars[0]);
     // let's parse second number
     if (n2x > 0) {
       thparse_double(sv,n2,pars[n2x]);
       if ((sv != TT_SV_NUMBER) || (n2 <= 0.0))
-        ththrow(("real positive number required -- %s", pars[0]));
+        ththrow("real positive number required -- {}", pars[0]);
       n1 = n2 / n1;
+      if (ux > 0)
+        n1 = lentf.transform(n1);
     }
+    this->scale = n1;
   }    
-  // let's parse units
-  if (ux > 0) {
-    lentf.parse_units(pars[ux]);
-    if (p9) {
-      this->scale_r1x = lentf.transform(this->scale_r1x);
-      this->scale_r1y = lentf.transform(this->scale_r1y);
-      this->scale_r2x = lentf.transform(this->scale_r2x);
-      this->scale_r2y = lentf.transform(this->scale_r2y);
-    } 
-    else {
-      n1 = lentf.transform(n1);
-    }
-  }
-  
-  this->scale = n1;
 }
 
 thdb2dcp * thscrap::insert_control_point()
@@ -377,7 +387,7 @@ thscraplo * thscrap::get_outline() {
         lo = co;
       } else {
 			  // vzdialenost moojho konca od zaciatku
-			  mindist = hypot(co->line->last_point->point->x - 
+			  mindist = std::hypot(co->line->last_point->point->x - 
 					co->line->first_point->point->x, 
 					co->line->last_point->point->y - 
 					co->line->first_point->point->y);
@@ -391,7 +401,7 @@ thscraplo * thscrap::get_outline() {
 					pco3 = &(this->outline_first);
 					while (co3 != NULL) {
 						if ((co3->line->outline == co->line->outline) &&
-								(hypot(co3->line->last_point->point->x - 
+								(std::hypot(co3->line->last_point->point->x - 
 										co3->line->first_point->point->x, 
 										co3->line->last_point->point->y - 
 										co3->line->first_point->point->y) < mindist)) {
@@ -412,6 +422,62 @@ thscraplo * thscrap::get_outline() {
     obj = obj->nscrapoptr;
   }
   
+  co = this->outline_first;
+  while (co != NULL) {
+    if (co->line->is_closed) {
+      co->lfreefirst = false;
+      co->lfreelast = false;
+    }
+    co2 = co->next_scrap_line;
+    while (co2 != NULL && (co->lfreefirst || co->lfreelast)) {
+      if ((co2->line->outline == co->line->outline) && (co2->lfreefirst || co2->lfreelast)) {
+        if (co->lfreefirst && co2->lfreefirst && 
+          (co->line->first_point->point->x == co2->line->first_point->point->x) && 
+          (co->line->first_point->point->y == co2->line->first_point->point->y)) {
+            co->lfreefirst = false;
+            co2->lfreefirst = false;
+#ifdef THDEBUG
+            thprintf("\tJOIN F-F: %.2f,%.2f\n", co->line->first_point->point->x, co->line->first_point->point->y);
+            thprintf("\t%d -- %d\n", co->line->id, co2->line->id);
+#endif
+        }
+        if (co->lfreefirst && co2->lfreelast && 
+          (co->line->first_point->point->x == co2->line->last_point->point->x) && 
+          (co->line->first_point->point->y == co2->line->last_point->point->y)) {
+            co->lfreefirst = false;
+            co2->lfreelast = false;
+#ifdef THDEBUG
+            thprintf("\tJOIN F-L: %.2f,%.2f\n", co->line->first_point->point->x, co->line->first_point->point->y);
+            thprintf("\t%d -- %d\n", co->line->id, co2->line->id);
+#endif
+        }
+        if (co->lfreelast && co2->lfreefirst && 
+          (co->line->last_point->point->x == co2->line->first_point->point->x) && 
+          (co->line->last_point->point->y == co2->line->first_point->point->y)) {
+            co->lfreelast = false;
+            co2->lfreefirst = false;
+#ifdef THDEBUG
+            thprintf("\tJOIN L-F: %.2f,%.2f\n", co->line->last_point->point->x, co->line->last_point->point->y);
+            thprintf("\t%d -- %d\n", co->line->id, co2->line->id);
+#endif
+        }
+        if (co->lfreelast && co2->lfreelast && 
+          (co->line->last_point->point->x == co2->line->last_point->point->x) && 
+          (co->line->last_point->point->y == co2->line->last_point->point->y)) {
+            co->lfreelast = false;
+            co2->lfreelast = false;
+#ifdef THDEBUG
+            thprintf("\tJOIN L-L: %.2f,%.2f\n", co->line->last_point->point->x, co->line->last_point->point->y);
+            thprintf("\t%d -- %d\n", co->line->id, co2->line->id);
+#endif
+        }
+      }
+      co2 = co2->next_scrap_line;
+    }
+    co = co->next_scrap_line;
+  }
+  
+
   bool still_in_line, co3_normal = false;
   thdb2dpt * first_pt, * last_pt;
   if (this->outline_first != NULL) {
@@ -425,30 +491,55 @@ thscraplo * thscrap::get_outline() {
 #endif
       still_in_line = !(co->line->is_closed);
       co3 = co;
+      bool search_all;
       while (still_in_line) {
-        mindist = hypot(last_pt->x - first_pt->x, last_pt->y - first_pt->y);
+        mindist = std::hypot(last_pt->x - first_pt->x, last_pt->y - first_pt->y);
         co2 = co->next_scrap_line;
+        search_all = false;
         co3last = co3;
         co3 = NULL;
         while (co2 != NULL) {
           if ((co2->mode == TT_OUTLINE_NO) &&
               (co2->line->outline == co->line->outline)) {
-            cdist = hypot(co2->line->last_point->point->x - last_pt->x,
-                          co2->line->last_point->point->y - last_pt->y);
-            if (cdist <= mindist) {
-              co3_normal = false;
-              co3 = co2;
-              mindist = cdist;
+            if ((co2->line->last_point->point->x == last_pt->x) && 
+                (co2->line->last_point->point->y == last_pt->y)) {
+                co3_normal = false;
+                co3 = co2;
+                mindist = 0.0;
             }
-            cdist = hypot(co2->line->first_point->point->x - last_pt->x,
-                          co2->line->first_point->point->y - last_pt->y);
-            if (cdist <= mindist) {
-              co3_normal = true;
-              co3 = co2;
-              mindist = cdist;
+            if (search_all || co2->lfreelast) {
+              cdist = std::hypot(co2->line->last_point->point->x - last_pt->x,
+                            co2->line->last_point->point->y - last_pt->y);
+              if (cdist <= mindist) {
+                co3_normal = false;
+                co3 = co2;
+                mindist = cdist;
+              }
+            }
+            if ((co2->line->first_point->point->x == last_pt->x) && 
+                (co2->line->first_point->point->y == last_pt->y)) {
+                co3_normal = true;
+                co3 = co2;
+                mindist = 0.0;
+            }
+            if (search_all || co2->lfreefirst) {
+              cdist = std::hypot(co2->line->first_point->point->x - last_pt->x,
+                            co2->line->first_point->point->y - last_pt->y);
+              if (cdist <= mindist) {
+                co3_normal = true;
+                co3 = co2;
+                mindist = cdist;
+              }
             }
           }
           co2 = co2->next_scrap_line;
+          if ((co2 == NULL) && (!search_all) && (co3 == NULL)) {
+            co2 = co->next_scrap_line;
+            search_all = true;
+#ifdef THDEBUG
+            thprintf("\tsearch all\n");
+#endif
+          }
         }
         if (co3 == NULL)
           still_in_line = false;
@@ -482,15 +573,25 @@ thscraplo * thscrap::get_outline() {
 } // get_outline
 
 
-thscraplp * thscrap::polygon_insert() {
+thscraplp * thscrap::polygon_insert(thscraplp * before) {
   thscraplp * rv = this->db->db2d.insert_scraplp();
-  if (this->polygon_last == NULL) {
-    this->polygon_first = rv;
-    this->polygon_last = rv;
+  if (before == NULL) {
+    if (this->polygon_last == NULL) {
+      this->polygon_first = rv;
+      this->polygon_last = rv;
+    } else {
+      this->polygon_last->next_item = rv;
+      rv->prev_item = this->polygon_last;
+      this->polygon_last = rv;
+    }
   } else {
-    this->polygon_last->next_item = rv;
-    rv->prev_item = this->polygon_last;
-    this->polygon_last = rv;
+    if (before->prev_item == NULL)
+      this->polygon_first = rv;
+    else
+      before->prev_item->next_item = rv;
+    rv->prev_item = before->prev_item;
+    before->prev_item = rv;
+    rv->next_item = before;
   }
   return rv;
 }
@@ -529,7 +630,7 @@ thscraplp * thscrap::get_polygon() {
   thdb2dcp * cp;
   thscraplp * lp, * nlp;
   thdb1ds * st, * st1, * st2;
-  unsigned long i, ni = this->db->db1d.station_vec.size(),
+  size_t i, ni = this->db->db1d.station_vec.size(),
     nlegs = this->db->db1d.leg_vec.size();
   thdataleg * cl;
   bool ffselect, ttselect;
@@ -553,14 +654,16 @@ thscraplp * thscrap::get_polygon() {
         }
         set_ffst_ttst;
         ffselect = ((this->centerline_survey == NULL) && (ffst->survey->selected));
-        ttselect = ((this->centerline_survey == NULL) && (ffst->survey->selected));
+        ttselect = ((this->centerline_survey == NULL) && (ttst->survey->selected));
         ffselect = ffselect || ((this->centerline_survey != NULL) && 
           (ffst->survey->is_in_survey(this->centerline_survey)));
         ttselect = ttselect || ((this->centerline_survey != NULL) && 
           (ttst->survey->is_in_survey(this->centerline_survey)));          
-        if (ffselect && ttselect && ((cl->extend & TT_EXTENDFLAG_IGNORE) == 0)) {
-          ffst->tmpselect = true;
-          ttst->tmpselect = true;
+        if ((cl->extend & TT_EXTENDFLAG_HIDE) == 0) {
+          if ((ffselect && ttselect) || (cl->psurvey->is_in_survey(this->centerline_survey))) {
+            ffst->tmpselect = true;
+            ttst->tmpselect = true;
+          }
         }
       }
     } else {
@@ -571,15 +674,26 @@ thscraplp * thscrap::get_polygon() {
           st->tmpselect = true;
         }
       }
+      for (i = 0; i < nlegs; i++) {
+        cl = this->db->db1d.leg_vec[i].leg;
+        //if (((cl->extend & TT_EXTENDFLAG_HIDE) == 0) && (cl->psurvey->is_in_survey(this->centerline_survey))) {
+        if ((cl->psurvey->is_in_survey(this->centerline_survey))) {
+          this->db->db1d.station_vec[cl->from.id - 1].tmpselect = true;
+          this->db->db1d.station_vec[cl->to.id - 1].tmpselect = true;
+        }
+      }
     } // other projections
 
     // enter selected stations
     for(i = 0; i < ni; i++) {
       st = &(this->db->db1d.station_vec[i]);
-      if (st->tmpselect) {
+      if (st->tmpselect && !st->is_temporary()) {
         lp = this->polygon_insert();
         lp->station = st;
         lp->ustation = st;
+        lp->station_name.id = (unsigned long)i+1;
+        lp->station_name.name = st->name;
+        lp->station_name.psurvey = st->survey;
         this->insert_adata(st);
       }
     }  
@@ -604,31 +718,38 @@ thscraplp * thscrap::get_polygon() {
               switch (this->proj->type) {
                 case TT_2DPROJ_EXTEND:
                   cnode = &(nodes[st->uid - 1]);
-                  arrow = cnode->first_arrow;
-                  lp->stx = thnan;
+                  lp->stx = cnode->extendx - this->proj->shift_x;
                   lp->sty = st->z - this->proj->shift_z;
                   lp->stz = 0.0;
+                  arrow = cnode->first_arrow;
                   while (arrow != NULL) {
-                    if ((arrow->leg->leg->extend & TT_EXTENDFLAG_IGNORE) == 0) {
-                      newx = (arrow->is_reversed ? arrow->leg->leg->txx : arrow->leg->leg->fxx) - this->proj->shift_x;
-                      if (thisnan(lp->stx)) {
-                        lp->stx = newx;
-                        lp->sty = st->z - this->proj->shift_z;
-                        lp->stz = 0.0;
-                      } else if (incp && (lp->stx != newx)) {
-                        nlp = this->polygon_insert();
-                        nlp->station = st;
-                        nlp->ustation = st;
-                        nlp->stx = newx;
-                        nlp->sty = st->z - this->proj->shift_z;
-                        nlp->stz = 0.0;
-                      }
+                    newx = (arrow->is_reversed ? arrow->leg->leg->txx : arrow->leg->leg->fxx) - this->proj->shift_x;
+
+                    if (((arrow->leg->leg->extend & TT_EXTENDFLAG_HIDE) == 0) && (newx != lp->stx)) {
+                      nlp = this->polygon_insert(lp);
+                      nlp->station = st;
+                      nlp->ustation = st;
+                      nlp->station_name.id = st->uid;
+                      nlp->station_name.name = st->name;
+                      nlp->station_name.psurvey = st->survey;
+                      nlp->stx = newx - this->proj->shift_x;
+                      nlp->sty = st->z - this->proj->shift_z;
+                      nlp->stz = 0.0;
+                      thscraplp * olp;
+                      olp = nlp;
+
+                      nlp = this->polygon_insert(lp);
+                      nlp->lnio = true;
+                      nlp->type = SYML_MAPCONNECTION;
+                      nlp->lnx1 = lp->stx;
+                      nlp->lny1 = lp->sty;
+                      nlp->lnz1 = lp->stz;
+                      nlp->lnx2 = olp->stx;
+                      nlp->lny2 = olp->sty;
+                      nlp->lnz2 = olp->stz;
                     }
                     arrow = arrow->next_arrow;
                   }
-                  if (thisnan(lp->stx)) {
-                    lp->stx = cnode->xx - this->proj->shift_x;
-                  }
                   break;
                 case TT_2DPROJ_PLAN:
                   lp->stx = st->x - this->proj->shift_x;
@@ -643,6 +764,7 @@ thscraplp * thscrap::get_polygon() {
                             + cosa * (st->y - this->proj->shift_y);
                   break;
               }
+              lp->station_name.name = st->name;
               this->update_limits(lp->stx, lp->sty);
             }
           } else
@@ -653,15 +775,19 @@ thscraplp * thscrap::get_polygon() {
             arrow = nodes[st->uid - 1].first_arrow;
             while (arrow != NULL) {
             
-              if ((((this->centerline_survey != NULL) && (arrow->leg->survey->is_in_survey(this->centerline_survey))) || 
-                  ((this->centerline_survey == NULL) && (arrow->leg->survey->selected))) &&
-                  ((this->proj->type != TT_2DPROJ_EXTEND) || ((arrow->leg->leg->extend & TT_EXTENDFLAG_IGNORE) == 0))) {
+//              if ((((this->centerline_survey != NULL) && (arrow->leg->survey->is_in_survey(this->centerline_survey))) || 
+//                  ((this->centerline_survey == NULL) && (arrow->leg->survey->selected))) &&
+//                  ((this->proj->type != TT_2DPROJ_EXTEND) || ((arrow->leg->leg->extend & TT_EXTENDFLAG_IGNORE) == 0))) {
+              if (((((this->centerline_survey != NULL) && (arrow->leg->survey->is_in_survey(this->centerline_survey))) || 
+                  ((this->centerline_survey == NULL) && (arrow->leg->survey->selected)))) &&
+			      ((this->proj->type != TT_2DPROJ_EXTEND) || 
+				  ((arrow->leg->leg->extend & TT_EXTENDFLAG_HIDE) == 0))) {
 
                 // process arrow here
                 nlp = this->polygon_insert();
                 nlp->lnio = true;
                 nlp->arrow = arrow;
-                nlp->type = (arrow->leg->leg->flags & TT_LEGFLAG_SURFACE != 0 ? SYML_SURVEY_SURFACE : SYML_SURVEY_CAVE);
+                nlp->type = (((arrow->leg->leg->flags & TT_LEGFLAG_SURFACE) != 0) ? SYML_SURVEY_SURFACE : SYML_SURVEY_CAVE);
                 st1 = &(this->db->db1d.station_vec[arrow->start_node->uid - 1]);
                 st2 = &(this->db->db1d.station_vec[arrow->end_node->uid - 1]);
                 switch (this->proj->type) {
@@ -736,18 +862,22 @@ void thscrap::calc_z()
 {
   thdb2dcp * cp = this->fcpp;
   this->z = 0.0;
+  this->a = 0.0;
   unsigned long numcp = 0;
   while(cp != NULL) {
     if (cp->st != NULL) {
       this->z += cp->tz;
+      this->a += cp->ta;
       numcp++;
     }
     cp = cp->nextcp;
   }
   if (numcp > 0) {
     this->z /= double(numcp);
+    this->a /= double(numcp);
   } else {
     this->z = thnan;
+    this->a = thnan;
   }
 }
 
@@ -762,7 +892,7 @@ thdb1ds * thscrap::get_nearest_station(thdb2dpt * pt) {
   thdb2dcp * cp = this->fcpp;
   while(cp != NULL) {
     if (cp->st != NULL) {
-      cdist = hypot(cp->pt->xt - pt->xt,cp->pt->yt - pt->yt);
+      cdist = std::hypot(cp->pt->xt - pt->xt,cp->pt->yt - pt->yt);
       if ((cdist < mindist) || (mindist < 0.0)) {
         res = cp->st;
         mindist = cdist;
@@ -775,7 +905,7 @@ thdb1ds * thscrap::get_nearest_station(thdb2dpt * pt) {
   thscraplp * lp = this->polygon_first;
   while(lp != NULL) {
     if (lp->station != NULL) {
-      cdist = hypot(lp->stx - pt->xt,lp->sty - pt->yt);
+      cdist = std::hypot(lp->stx - pt->xt,lp->sty - pt->yt);
       if ((cdist < mindist) || (mindist < 0.0)) {
         res = lp->station;
         mindist = cdist;
@@ -922,16 +1052,18 @@ thscrapen * thscrap::get_ends() {
 
 void thscrap::insert_adata(class thdb1ds * as) {
   // najde ci dane data uz mame
-  if (as->data == NULL)
+  if (as->data.empty())
     return;
   thmapstat_dataptr temp;
-  temp.ptr = as->data;
-  
-  thmapstat_datamap::iterator di = this->adata.find(temp);
-  if (di == adata.end()) {
-    this->adata[temp] = 1;
-  } else {
-    di->second++;
+
+  for(std::list<thdata *>::iterator it = as->data.begin(); it != as->data.end(); it++) {
+	  temp.ptr = *it;
+	  thmapstat_datamap::iterator di = this->adata.find(temp);
+	  if (di == adata.end()) {
+		this->adata[temp] = 1;
+	  } else {
+		di->second++;
+	  }
   }
   
 }
@@ -944,13 +1076,18 @@ void thscrap::process_3d() {
     
   this->d3_parsed = true;
 
-  thprintf("[%s] ", this->name);
+  if (this->d3 == TT_FALSE)
+	  return;
 
-  thscrapis is;
+  thprintf(".");
+
+  thscrapis is(this);
+
+#ifndef THSCRAPIS_NEW3D
   thscraplp * slp;
   th2ddataobject * o2;
   thpoint * pp;
-  
+
   // vlozi vsetky meracske body (body + body z polygonu)
   slp = this->get_polygon();
   while (slp != NULL) {
@@ -995,38 +1132,11 @@ void thscrap::process_3d() {
     }
     slp = slp->next_item;
   }
-  
-  // vypise siet pre scrap
-  
-//  double iddx = (this->lxmax - this->lxmin) / 20.0, 
-//    iddy = (this->lymax - this->lymin) / 20.0,
-//    iz[21][21], id, idx, idy;
-//  long i, j;
-//  for (i = 0; i <= 20; i++) {
-//    for (j = 0; j <= 20; j++) {
-//      is.bp_interpolate(this->lxmin + iddx * i, this->lymin + iddy * j,
-//        iz[i][j], id, idx, idy);
-//    }
-//  }
-//
-//  for (j = 0; j < 20; j++) {
-//    fprintf(out,"glColor3f 0.5 0.5 0.5\n");
-//    fprintf(out,"glBegin $GL::GL_TRIANGLE_STRIP\n");
-//    for(i = 0; i <= 20; i++) {
-//      fprintf(out,"glVertex3f %.2f\t%2f\t%.2f\n",
-//        this->lxmin + this->proj->shift_x + iddx * i - avx,
-//        this->lymin + this->proj->shift_y + iddy * j - avy,
-//        iz[i][j] + this->proj->shift_z - avz);
-//      fprintf(out,"glVertex3f %.2f\t%2f\t%.2f\n",
-//        this->lxmin + this->proj->shift_x + iddx * i - avx,
-//        this->lymin + this->proj->shift_y + iddy * (j + 1) - avy,
-//        iz[i][j + 1] + this->proj->shift_z - avz);
-//    }
-//    fprintf(out,"glEnd\n");
-//  }
-  
+#endif
+
   is.outline_scan(this->get_outline());
 
+#ifndef THSCRAPIS_NEW3D
   // prejde vsetky objekty a nasackuje passage-heights
   o2 = this->fs2doptr;
   double cup, cdown;
@@ -1049,12 +1159,15 @@ void thscrap::process_3d() {
     o2 = o2->nscrapoptr;
   }
   is.insert_bp_dim();
-  is.outline_interpolate_dims();  
+  is.outline_interpolate_dims(); 
+#endif
   
   thscrapisolpt * oline, * olineln, * prevolineln = NULL;
-  double normx, normy, norml;
   bool started = false;
 
+#ifndef THSCRAPIS_NEW3D
+  double normx, normy, norml;
+
   // povkladame vsetky body na outlineoch
   for(oline = is.firstolseg; oline != NULL; oline = oline->next_segment) {
     for(olineln = oline; olineln != NULL; olineln = olineln->next) {
@@ -1069,14 +1182,6 @@ void thscrap::process_3d() {
     }
   }
 
-//    fprintf(out,"glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE {0.0 0.8 0.0 1.0}\n");
-//    fprintf(out,"glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT {0.0 0.0 0.0 1.0}\n");
-//    fprintf(out,"glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR {0.0 0.0 0.0 1.0}\n");
-//    fprintf(out,"glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS {0.0}\n");
-//    fprintf(out,"glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION {0.0 0.0 0.0 1.0}\n");
-//    fprintf(out,"glBegin $GL::GL_TRIANGLE_STRIP\n");
-//    fprintf(out,"glNormal3f %.2f\t%.2f\t%.2f\n", normx / norml, normy / norml, 0.0);
-
   oline = is.firstolseg;
   thdb3dfc * cfc = NULL;
   thdb3dfx * cfx, * cfx2;
@@ -1087,7 +1192,7 @@ void thscrap::process_3d() {
       if (EXPORT3D_INVISIBLE || olineln->visible) {
         normx = olineln->y - olineln->prev->y;
         normy = olineln->prev->x - olineln->x;
-        norml = hypot(normx, normy);
+        norml = std::hypot(normx, normy);
         if (!started) {
           cfc = this->d3_outline.insert_face(THDB3DFC_TRIANGLE_STRIP);
           cfx = cfc->insert_vertex(olineln->prev->vx3dup);
@@ -1126,7 +1231,7 @@ void thscrap::process_3d() {
       if (prevolineln != NULL) {
         normx = oline->y - prevolineln->y;
         normy = prevolineln->x - oline->x;
-        norml = hypot(normx, normy);
+        norml = std::hypot(normx, normy);
       } else {
         normx = 1.0;
         normy = 1.0;
@@ -1163,22 +1268,22 @@ void thscrap::process_3d() {
     cfc = this->d3_outline.insert_face(THDB3DFC_TRIANGLES);
     for(i = 0; i < is.tri_num; i++) {
 
-      oline = is.tri_opts[is.tri_triangles[i][0]];
-      olineln = is.tri_opts[is.tri_triangles[i][1]];
+      oline = is.tri_triangles[i][0];
+      olineln = is.tri_triangles[i][1];
       vx = olineln->x - oline->x;
       vy = olineln->y - oline->y;
       vz = olineln->zd - oline->zd;
-      oline = is.tri_opts[is.tri_triangles[i][1]];
-      olineln = is.tri_opts[is.tri_triangles[i][2]];
+      oline = is.tri_triangles[i][1];
+      olineln = is.tri_triangles[i][2];
       wx = olineln->x - oline->x;
       wy = olineln->y - oline->y;
       wz = olineln->zd - oline->zd;
       nx = - (vy * wz - wy * vz);
       ny = - (wx * vz - vx * wz);
       nz = - (vx * wy - wx * vy);
-      nl = hypot(hypot(nx, ny), nz);
+      nl = std::hypot(std::hypot(nx, ny), nz);
       for(j = 2; j >= 0; j--) {
-        oline = is.tri_opts[is.tri_triangles[i][j]];
+        oline = is.tri_triangles[i][j];
         cfx = cfc->insert_vertex(oline->vx3ddn);
         if (nl > 0.0) {
           if (j == 2)
@@ -1187,22 +1292,22 @@ void thscrap::process_3d() {
         }
       }
       
-      oline = is.tri_opts[is.tri_triangles[i][0]];
-      olineln = is.tri_opts[is.tri_triangles[i][1]];
+      oline = is.tri_triangles[i][0];
+      olineln = is.tri_triangles[i][1];
       vx = olineln->x - oline->x;
       vy = olineln->y - oline->y;
       vz = olineln->zu - oline->zu;
-      oline = is.tri_opts[is.tri_triangles[i][1]];
-      olineln = is.tri_opts[is.tri_triangles[i][2]];
+      oline = is.tri_triangles[i][1];
+      olineln = is.tri_triangles[i][2];
       wx = olineln->x - oline->x;
       wy = olineln->y - oline->y;
       wz = olineln->zu - oline->zu;
       nx = (vy * wz - wy * vz);
       ny = (wx * vz - vx * wz);
       nz = (vx * wy - wx * vy);
-      nl = hypot(hypot(nx, ny), nz);
+      nl = std::hypot(std::hypot(nx, ny), nz);
       for(j = 0; j < 3; j++) {
-        oline = is.tri_opts[is.tri_triangles[i][j]];
+        oline = is.tri_triangles[i][j];
         cfx = cfc->insert_vertex(oline->vx3dup);
         if (nl > 0.0) {
           if (j == 0)
@@ -1213,6 +1318,118 @@ void thscrap::process_3d() {
     }
     
   }
+#endif
+
+
+#ifdef THSCRAPIS_NEW3D
+  lxVec norm;
+  double norml;
+
+  // povkladame horne a spodne body
+  for(oline = is.firstolseg; oline != NULL; oline = oline->next_segment) {
+    for(olineln = oline; olineln != NULL; olineln = olineln->next) {
+      olineln->vx3dup = this->d3_outline.insert_vertex(olineln->pt + olineln->d_up * olineln->dir);
+      olineln->vx3ddn = this->d3_outline.insert_vertex(olineln->pt - olineln->d_dn * olineln->dir);
+    }
+  }
+
+  // vlozime triangle strip dookola
+  oline = is.firstolseg;
+  thdb3dfc * cfc = NULL;
+  thdb3dfx * cfx, * cfx2;
+  while (oline != NULL) {
+
+    started = false;
+
+    // najdeme posledny bod
+    prevolineln = NULL;
+    olineln = oline->next;
+    while (olineln != NULL) {
+      prevolineln = olineln;
+      olineln = olineln->next;
+    }
+    // nemame ani 2 body - nerobime outline
+    if (prevolineln == NULL) 
+      break;
+
+    olineln = oline;
+    while (olineln != NULL) {
+      if (EXPORT3D_INVISIBLE || olineln->visible) {
+        norm = (olineln->pt - prevolineln->pt) ^ (olineln->dir);
+        norml = norm.Length();
+        if (!started) {
+          cfc = this->d3_outline.insert_face(THDB3DFC_TRIANGLE_STRIP);
+          cfx = cfc->insert_vertex(prevolineln->vx3dup);
+          cfx2 = cfc->insert_vertex(prevolineln->vx3ddn);
+          if (norml > 0.0) {
+            norm /= norml;
+            cfx->insert_normal(norm);
+            cfx->vertex->insert_normal(2.0 * norm);
+            cfx2->vertex->insert_normal(2.0 * norm);
+          }
+          started = true;
+        }
+        cfx = cfc->insert_vertex(olineln->vx3dup);
+        cfx2 = cfc->insert_vertex(olineln->vx3ddn);
+        if (norml > 0.0) {
+          cfx->insert_normal(norm);
+          cfx->vertex->insert_normal(2.0 * norm);
+          cfx2->vertex->insert_normal(2.0 * norm);
+        }
+      } else {
+        if (started) {
+          started = false;
+        }
+      }
+      prevolineln = olineln;
+      olineln = olineln->next;
+    }
+    
+    // prejdeme dalsi segment
+    oline = oline->next_segment;
+  }
+  
+  int i, j;
+  lxVec v, w;
+  if (is.tri_num > 0) {    
+    cfc = this->d3_outline.insert_face(THDB3DFC_TRIANGLES);
+    for(i = 0; i < is.tri_num; i++) {
+
+      // spodne trojuholniky
+      v = is.tri_triangles[i][1]->vx3ddn->get_vector() - is.tri_triangles[i][0]->vx3ddn->get_vector();
+      w = is.tri_triangles[i][2]->vx3ddn->get_vector() - is.tri_triangles[i][1]->vx3ddn->get_vector();
+      norm = -1.0 * (v ^ w);
+      norml = norm.Length();
+      if (norml > 0.0) norm /= norml;
+      for(j = 2; j >= 0; j--) {
+        oline = is.tri_triangles[i][j];
+        cfx = cfc->insert_vertex(oline->vx3ddn);
+        if (norml > 0.0) {
+          if (j == 2)
+            cfx->insert_normal(norm);
+          cfx->vertex->insert_normal(norm);
+        }
+      }
+
+      // horne trojuholniky
+      v = is.tri_triangles[i][1]->vx3dup->get_vector() - is.tri_triangles[i][0]->vx3dup->get_vector();
+      w = is.tri_triangles[i][2]->vx3dup->get_vector() - is.tri_triangles[i][1]->vx3dup->get_vector();
+      norm = (v ^ w);
+      norml = norm.Length();
+      if (norml > 0.0) norm /= norml;
+      for(j = 0; j < 3; j++) {
+        oline = is.tri_triangles[i][j];
+        cfx = cfc->insert_vertex(oline->vx3dup);
+        if (norml > 0.0) {
+          if (j == 0)
+            cfx->insert_normal(norm);
+          cfx->vertex->insert_normal(norm);
+        }
+      }
+    }
+  }
+#endif
+
 
   this->d3_outline.postprocess();
   
@@ -1245,3 +1462,52 @@ void thscrap::update_limits(double x, double y)
 }
 
 
+
+void thscrap::parse_sketch(char ** args, int argenc)
+{
+  int sv;
+  thsketch sk;
+  sk.m_scrap = this;
+  sk.m_pic.init(args[0], thdb.csrc.name);
+  // X
+  thparse_double(sv,sk.m_x,args[1]);
+  if ((sv	!= TT_SV_NUMBER) &&	(sv	!= TT_SV_NAN))
+    ththrow("invalid	number --	{}", args[1]);
+  // Y
+  thparse_double(sv,sk.m_y,args[2]);
+  if ((sv	!= TT_SV_NUMBER) &&	(sv	!= TT_SV_NAN))
+    ththrow("invalid	number --	{}", args[2]);
+  this->sketch_list.push_back(std::move(sk));
+}
+
+
+void thscrap::start_insert() {
+  if (this->cs != TTCS_LOCAL) {
+    if (this->proj->type != TT_2DPROJ_PLAN)
+      ththrow("coordinate system specification valid only for plan projection");
+    if (!this->scale_p9)
+      ththrow("scrap scaling not valid in this coordinate system");
+  }
+}
+
+
+void thscrap::convert_all_cs() {
+    if (this->scale_cs != TTCS_LOCAL) {
+        this->convert_cs(this->scale_cs, this->scale_r1x, this->scale_r1y, this->scale_r1x, this->scale_r1y);
+        this->convert_cs(this->scale_cs, this->scale_r2x, this->scale_r2y, this->scale_r2x, this->scale_r2y);
+	}
+}
+
+
+void thscrap::add_joined_scrap_stations(thscrap * js) {
+  thdb2dcp * cp = js->fcpp;
+  while(cp != NULL) {
+	if (cp->st != NULL)
+	  this->joined_scrap_stations.insert(cp);
+	cp = cp->nextcp;
+  }	
+}
+
+
+
+
