@@ -38,6 +38,10 @@
 #include "thscrapen.h"
 #include "thmapstat.h"
 #include "thdb3d.h"
+#include "thsketch.h"
+#include "thtrans.h"
+
+#include <set>
 
 /**
  * scrap command options tokens.
@@ -50,6 +54,7 @@ enum {
   TT_SCRAP_STATIONS = 2003,
   TT_SCRAP_3D = 2004,
   TT_SCRAP_FLIP = 2005,
+  TT_SCRAP_SKETCH = 2006,
 };
 
 enum {
@@ -69,6 +74,7 @@ static const thstok thtt_scrap_opt[] = {
   {"proj", TT_SCRAP_PROJECTION},
   {"projection", TT_SCRAP_PROJECTION},
   {"scale", TT_SCRAP_SCALE},
+  {"sketch", TT_SCRAP_SKETCH},
   {"stations", TT_SCRAP_STATIONS},
   {"walls", TT_SCRAP_3D},
   {NULL, TT_SCRAP_UNKNOWN},
@@ -89,11 +95,15 @@ class thscrap : public thdataobject {
   class th2ddataobject * fs2doptr,  ///< First scrap 2D object.
     * ls2doptr;  ///< Last scrap 2D object.
     
-  double lxmin, lxmax, lymin, lymax, ///< Coordinate limits.
-    R, G, B; ///< Scrap color.
+  double lxmin, lxmax, lymin, lymax; ///< Coordinate limits.
+
+  unsigned long RGBsrc;
+
+  thlayout_color clr; ///< Scrap color;
     
   thdb2dcp * fcpp, * lcpp; ///< Scrap control points.
   unsigned int ncp; ///< Number of control points.
+  std::set<thdb2dcp *> joined_scrap_stations; ///< Stations of joined scraps.
 
   class thsurvey * centerline_survey;
   bool centerline_io;
@@ -105,9 +115,10 @@ class thscrap : public thdataobject {
   
   double scale, scale_r1x, scale_r1y, scale_p1x, scale_p1y,
     scale_r2x, scale_r2y, scale_p2x, scale_p2y;  ///< Scrap scale.
+  int scale_cs;
   bool scale_p9; ///< 9 parameters scaling
   
-  double mx, my, mxx, mxy, myx, myy, mr, ms;  ///< Calibration coeficients.
+  double mx, my, mxx, mxy, myx, myy, mr, ms;  ///< Calibration coefficients.
   
   double maxdist, avdist;
   thdb2dpt * maxdistp1, * maxdistp2;
@@ -118,6 +129,7 @@ class thscrap : public thdataobject {
   thscrapen * ends_first;
   
   thmapstat_datamap adata;
+  thmorph2trans morph;
   
   int d3, flip;
   thdb3ddata d3_outline;
@@ -125,11 +137,13 @@ class thscrap : public thdataobject {
   void parse_scale(char * ss);
   void parse_stations(char * ss);
   void process_3d();
-  thscraplp * polygon_insert();
+  thscraplp * polygon_insert(thscraplp * before = NULL);
   
   thdb1ds * get_nearest_station(thdb2dpt * pt);
+
+  thsketch_list sketch_list;
   
-  double z;
+  double z, a;
     
   public:
 
@@ -151,76 +165,81 @@ class thscrap : public thdataobject {
    * Return class identifier.
    */
   
-  virtual int get_class_id();
+  int get_class_id() override;
   
   
   /**
    * Return class name.
    */
    
-  virtual char * get_class_name() {return "thscrap";};
+  const char * get_class_name() override {return "thscrap";};
   
   
   /**
    * Return true, if son of given class.
    */
   
-  virtual bool is(int class_id);
+  bool is(int class_id) override;
   
   
   /**
    * Return number of command arguments.
    */
    
-  virtual int get_cmd_nargs();
+  int get_cmd_nargs() override;
   
   
   /**
    * Return command name.
    */
    
-  virtual char * get_cmd_name();
+  const char * get_cmd_name() override;
   
   
   /**
    * Return command end option.
    */
    
-  virtual char * get_cmd_end();
+  const char * get_cmd_end() override;
   
   
   /**
    * Return option description.
    */
    
-  virtual thcmd_option_desc get_cmd_option_desc(char * opts);
+  thcmd_option_desc get_cmd_option_desc(const char * opts) override;
   
   
   /**
    * Set command option.
    *
    * @param cod Command option description.
-   * @param args Option arguments arry.
+   * @param args Option arguments array.
    * @param argenc Arguments encoding.
    */
    
-  virtual void set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline);
+  void set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline) override;
 
 
   /**
-   * Delete this object.
-   *
-   * @warn Always use this methos instead of delete function.
+   * Parse sketch option.
    */
-   
-  virtual void self_delete();
+
+  void parse_sketch(char ** args, int argenc);
+
+
+  /**
+   * Create a sketch linked to current scrap.
+   */
+
+  void insert_sketch(char ** args);
 
 
   /**
    * Print object properties.
    */
    
-  virtual void self_print_properties(FILE * outf); 
+  void self_print_properties(FILE * outf) override; 
   
   
   /**
@@ -285,6 +304,21 @@ class thscrap : public thdataobject {
  
  void update_limits(double x, double y);
 
+ void start_insert() override;
+
+ /**
+  * Convert all points in object.
+  */
+
+ void convert_all_cs() override;
+ 
+ 
+ /**
+  * Add stations from joined scrap.
+  */
+ void add_joined_scrap_stations(thscrap * js);
+
+
 };
 
 
