@@ -31,11 +31,16 @@
 #include "thmbuffer.h"
 #include "thtflength.h"
 #include "thdatabase.h"
-#include <math.h>
+#include "thcsdata.h"
+#include <cmath>
 #include "thdatareader.h"
 #include "thparse.h"
 #include "thdb1d.h"
 #include "thinfnan.h"
+#ifdef THMSVC
+#include <direct.h>
+#define getcwd _getcwd
+#endif
 
 enum {
   TT_SURFACE_GFLIP_UNKNOWN,
@@ -90,14 +95,17 @@ thsurface::thsurface()
   this->s1.clear();
   this->s2.clear();
   this->d3dok = false;
+
+  this->pict_cs = TTCS_LOCAL;
+  this->grid_cs = TTCS_LOCAL;
+
     
 }
 
 
 thsurface::~thsurface()
 {
-  if (this->grid != NULL)
-    delete [] this->grid;
+  delete [] this->grid;
 }
 
 
@@ -122,21 +130,21 @@ int thsurface::get_cmd_nargs()
 }
 
 
-char * thsurface::get_cmd_end()
+const char * thsurface::get_cmd_end()
 {
   // insert endcommand if multiline command
   return "endsurface";
 }
 
 
-char * thsurface::get_cmd_name()
+const char * thsurface::get_cmd_name()
 {
   // insert command name here
   return "surface";
 }
 
 
-thcmd_option_desc thsurface::get_cmd_option_desc(char * opts)
+thcmd_option_desc thsurface::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_surface_opt);
   if (id == TT_SURFACE_UNKNOWN)
@@ -173,10 +181,10 @@ void thsurface::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lo
     
     case TT_SURFACE_GRID_FLIP:
       if (this->grid != NULL)
-        ththrow(("grid-flip specification after grid data not allowed"));
+        ththrow("grid-flip specification after grid data not allowed");
       this->grid_flip = thmatch_token(args[0], thtt_surface_gflip);
       if (this->grid_flip == TT_SURFACE_GFLIP_UNKNOWN)
-        ththrow(("unknown surface flip mode -- %s", args[0]))
+        ththrow("unknown surface flip mode -- {}", args[0]);
       break;
       
     case TT_SURFACE_PICTURE:
@@ -190,11 +198,6 @@ void thsurface::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lo
 }
 
 
-void thsurface::self_delete()
-{
-  delete this;
-}
-
 void thsurface::self_print_properties(FILE * outf)
 {
   thdataobject::self_print_properties(outf);
@@ -210,15 +213,17 @@ void thsurface::parse_picture(char ** args)
 
   thbuffer pict_path;
   pict_path.guarantee(1024);
-  getcwd(pict_path.get_buffer(),1024);
-  
+  thassert(getcwd(pict_path.get_buffer(),1024) != NULL);  
   long i;
   if (strlen(args[0]) == 0)
-    ththrow(("picture name not specified"))
-  pict_path += "/";
-  pict_path += thdb.csrc.name;
+    ththrow("picture name not specified");
+  pict_path += "/";  
+  if (thpath_is_absolute(thdb.csrc.name))
+	  pict_path = thdb.csrc.name;
+  else
+	  pict_path += thdb.csrc.name;
   char * pp = pict_path.get_buffer();
-  for(i = strlen(pp); i >= 0; i--) {
+  for(i = (long)strlen(pp); i >= 0; i--) {
     if ((pp[i] == '/') || (pp[i] == '\\')) {
       break;
     } else
@@ -228,7 +233,7 @@ void thsurface::parse_picture(char ** args)
     pict_path = "/";
   pict_path += args[0];
   pp = pict_path.get_buffer();
-  for(i = strlen(pp); i >= 0; i--) {
+  for(i = (long)strlen(pp); i >= 0; i--) {
     if (pp[i] == '\\')
       pp[i] = '/';
   }
@@ -242,18 +247,17 @@ void thsurface::parse_picture(char ** args)
   // potom obrazok skontroluje, vytiahne jeho rozmery a DPI
   try {
     thparse_image(this->pict_name, this->pict_width, this->pict_height, this->pict_dpi, this->pict_type);
-  } catch (...) {
+  } catch (const std::exception& e) {
     this->pict_name = NULL;
-    threwarning(("%s [%d] -- error reading bitmap",
+    thwarning(("%s [%d] -- error reading bitmap -- %s",
       thdbreader.get_cinf()->get_cif_name(),
-      thdbreader.get_cinf()->get_cif_line_number()));
+      thdbreader.get_cinf()->get_cif_line_number(), e.what()));
   }
   
   // potom parsne kalibraciu
   thmbuffer calib;
   thsplit_words(&calib, args[1]);
   thtflength ltr;
-  double dblv;
   int sv;
   long ncals = calib.get_size();
   char ** cals = calib.get_buffer();
@@ -264,23 +268,29 @@ void thsurface::parse_picture(char ** args)
     case 8:
       // parsne jednotlive cisla
 #define surfpiccaldbl(XXX,YYY) \
-      thparse_double(sv, dblv, cals[YYY]); \
+      thparse_double(sv, this->XXX, cals[YYY]); \
       if (sv != TT_SV_NUMBER) \
-        ththrow(("number expected -- %s", cals[YYY])) \
-      this->XXX = ltr.transform(dblv);
+        ththrow("number expected -- {}", cals[YYY]);
       
       surfpiccaldbl(pict_X1,0);
       surfpiccaldbl(pict_Y1,1);
-      surfpiccaldbl(pict_x1,2);
-      surfpiccaldbl(pict_y1,3);
+      this->pict_cs = this->cs;
+      this->read_cs(cals[2], cals[3], this->pict_x1, this->pict_y1);
+      if (this->cs == TTCS_LOCAL) {
+        this->pict_x1 = ltr.transform(this->pict_x1);
+        this->pict_y1 = ltr.transform(this->pict_y1);
+      }
       surfpiccaldbl(pict_X2,4);
       surfpiccaldbl(pict_Y2,5);
-      surfpiccaldbl(pict_x2,6);
-      surfpiccaldbl(pict_y2,7);
+      this->read_cs(cals[6], cals[7], this->pict_x2, this->pict_y2);
+      if (this->cs == TTCS_LOCAL) {
+        this->pict_x2 = ltr.transform(this->pict_x2);
+        this->pict_y2 = ltr.transform(this->pict_y2);
+      }
       
       if (((this->pict_X1 == this->pict_X2) && (this->pict_Y1 == this->pict_Y2)) ||
           ((this->pict_x1 == this->pict_x2) && (this->pict_y1 == this->pict_y2))) {
-        ththrow(("duplicate points in picture calibration"));
+        ththrow("duplicate points in picture calibration");
       }
       break;
     
@@ -293,12 +303,12 @@ void thsurface::parse_picture(char ** args)
       thparse_objectname(this->s2, &(thdb.buff_stations), cals[5]);
       this->ssurvey = thdb.get_current_survey();
       if ((this->pict_X1 == this->pict_X2) && (this->pict_Y1 == this->pict_Y2)) {
-        ththrow(("duplicate points in picture calibration"));
+        ththrow("duplicate points in picture calibration");
       }
       break;
 
     default:
-      ththrow(("invalid number of picture calibration arguments -- %d", ncals))
+      ththrow("invalid number of picture calibration arguments -- {}", ncals);
   }
 
 }
@@ -307,9 +317,9 @@ void thsurface::parse_picture(char ** args)
 void thsurface::calibrate() {
   // spocita origin, scale (pri 300 dpi) a rotaciu
   double olen, nlen, scale, ang, tang;
-  olen = hypot(this->pict_X2 - this->pict_X1, 
+  olen = std::hypot(this->pict_X2 - this->pict_X1, 
                 this->pict_Y2 - this->pict_Y1);
-  nlen = hypot(this->pict_x2 - this->pict_x1, 
+  nlen = std::hypot(this->pict_x2 - this->pict_x1, 
                 this->pict_y2 - this->pict_y1);
   ang = atan2(this->pict_Y2 - this->pict_Y1, this->pict_X2 - this->pict_X1);
   tang = atan2(this->pict_y2 - this->pict_y1, this->pict_x2 - this->pict_x1);
@@ -341,20 +351,18 @@ void thsurface::check_stations()
     // najde stations, error ak nie
     this->s1.id = thdb.db1d.get_station_id(this->s1, this->ssurvey);
     if (this->s1.id == 0) {
-      this->throw_source();
       if (this->s1.survey == NULL)
-        threthrow2(("station doesn't exist -- %s", this->s1.name))
+        ththrow("{} -- station doesn't exist -- {}", this->throw_source(), this->s1.name);
       else
-        threthrow2(("station doesn't exist -- %s@%s", this->s1.name, this->s1.survey))
+        ththrow("{} -- station doesn't exist -- {}@{}", this->throw_source(), this->s1.name, this->s1.survey);
     }
     
     this->s2.id = thdb.db1d.get_station_id(this->s2, this->ssurvey);
     if (this->s2.id == 0) {
-      this->throw_source();
       if (this->s2.survey == NULL)
-        threthrow2(("station doesn't exist -- %s", this->s2.name))
+        ththrow("{} -- station doesn't exist -- {}", this->throw_source(), this->s2.name);
       else
-        threthrow2(("station doesn't exist -- %s@%s", this->s2.name, this->s2.survey))
+        ththrow("{} -- station doesn't exist -- {}@{}", this->throw_source(), this->s2.name, this->s2.survey);
     }
     
     // priradi si x a y a skontroluje ci su roozne
@@ -365,8 +373,7 @@ void thsurface::check_stations()
     this->pict_x2 = ds2->x;
     this->pict_y2 = ds2->y;
     if ((this->pict_x1 == this->pict_x2) && (this->pict_y1 == this->pict_y2)) {
-      this->throw_source();
-      threthrow2(("duplicate points in picture calibration"));
+      ththrow("{} -- duplicate points in picture calibration", this->throw_source());
     }
   }
 }
@@ -375,7 +382,7 @@ void thsurface::check_stations()
 void thsurface::parse_grid_setup(char ** args)
 {
   if (this->grid != NULL)
-    ththrow(("grid specification after grid data not allowed"));
+    ththrow("grid specification after grid data not allowed");
 
   // nacitame vsetky premenne ktore treba
   double dblv;
@@ -384,38 +391,43 @@ void thsurface::parse_grid_setup(char ** args)
 #define parsedbl(XXX,YYY) \
       thparse_double(sv, dblv, args[YYY]); \
       if (sv != TT_SV_NUMBER) \
-        ththrow(("number expected -- %s", args[YYY])) \
+        ththrow("number expected -- {}", args[YYY]); \
       XXX = this->grid_units.transform(dblv);
 #define parsenum(XXX,YYY) \
       thparse_double(sv, dblv, args[YYY]); \
       if (sv != TT_SV_NUMBER) \
-        ththrow(("number expected -- %s", args[YYY])) \
+        ththrow("number expected -- {}", args[YYY]); \
       if (dblv <= 0) \
-        ththrow(("positive number expected -- %s", args[YYY])) \
+        ththrow("positive number expected -- {}", args[YYY]); \
       if (dblv != double(long(dblv))) \
-        ththrow(("integer expected -- %s", args[YYY])) \
+        ththrow("integer expected -- {}", args[YYY]); \
       XXX = long(dblv);
-      
-  parsedbl(this->grid_ox,0);
-  parsedbl(this->grid_oy,1);
+  
+  this->grid_cs = this->cs;
+  this->read_cs(args[0], args[1], this->grid_ox, this->grid_oy);
+  if (this->cs == TTCS_LOCAL) {
+    this->grid_ox = this->grid_units.transform(this->grid_ox);
+    this->grid_oy = this->grid_units.transform(this->grid_oy);
+  }
+
   parsedbl(this->grid_dx,2);
   if (this->grid_dx == 0.0)
-    ththrow(("non-zero number expected -- %s", args[2]));
+    ththrow("non-zero number expected -- {}", args[2]);
   parsedbl(this->grid_dy,3);
   if (this->grid_dy == 0.0)
-    ththrow(("non-zero number expected -- %s", args[3]));
+    ththrow("non-zero number expected -- {}", args[3]);
   parsenum(this->grid_nx,4);
   if (this->grid_nx < 2)
-    ththrow(("number > 1 expected -- %s", args[4]));
+    ththrow("number > 1 expected -- {}", args[4]);
   parsenum(this->grid_ny,5);
   if (this->grid_ny < 2)
-    ththrow(("number > 1 expected -- %s", args[5]));
+    ththrow("number > 1 expected -- {}", args[5]);
 }
 
 void thsurface::parse_grid(char * spec)
 {
   if (this->grid_nx == 0)
-    ththrow(("grid dimensions not specified"));
+    ththrow("grid dimensions not specified");
   if (this->grid == NULL) {
     this->grid_counter = 0;
     this->grid_size = this->grid_nx * this->grid_ny;
@@ -434,9 +446,9 @@ void thsurface::parse_grid(char * spec)
   for(i = 0; i < ni; i++) {
     thparse_double(sv, alt, heights[i]);
     if (sv != TT_SV_NUMBER)
-      ththrow(("number expected -- %s", heights[i]))
+      ththrow("number expected -- {}", heights[i]);
     if (this->grid_counter == this->grid_size)
-      ththrow(("too many grid data"))
+      ththrow("too many grid data");
     x = this->grid_counter % this->grid_nx;
     y = this->grid_ny - (this->grid_counter / this->grid_nx) - 1;
     switch (this->grid_flip) {
@@ -454,7 +466,7 @@ void thsurface::parse_grid(char * spec)
 
 void thsurface::start_insert() {
   if (this->grid_counter < this->grid_size)
-    ththrow(("missing grid data"))
+    ththrow("missing grid data");
 }
 
 thdb3ddata * thsurface::get_3d() {
@@ -489,7 +501,7 @@ thdb3ddata * thsurface::get_3d() {
         nx = pvx->x - cvx->x;
         ny = pvx->y - cvx->y;
         nz = pvx->z - cvx->z;
-        nl = hypot(nz, hypot(nx, ny));
+        nl = std::hypot(nz, std::hypot(nx, ny));
         nx /= nl; ny /= nl; nz /= nl;
         nt = nx; nx = nz; nz = -nt;
         cvx->insert_normal(nx, ny, nz);
@@ -500,7 +512,7 @@ thdb3ddata * thsurface::get_3d() {
         nx = - pvx->x + cvx->x;
         ny = - pvx->y + cvx->y;
         nz = - pvx->z + cvx->z;
-        nl = hypot(nz, hypot(nx, ny));
+        nl = std::hypot(nz, std::hypot(nx, ny));
         nx /= nl; ny /= nl; nz /= nl;
         nt = nx; nx = nz; nz = -nt;
         cvx->insert_normal(nx, ny, nz);
@@ -511,7 +523,7 @@ thdb3ddata * thsurface::get_3d() {
         nx = pvx->x - cvx->x;
         ny = pvx->y - cvx->y;
         nz = pvx->z - cvx->z;
-        nl = hypot(nz, hypot(nx, ny));
+        nl = std::hypot(nz, std::hypot(nx, ny));
         nx /= nl; ny /= nl; nz /= nl;
         nt = ny; ny = nz; nz = -nt;
         cvx->insert_normal(nx, ny, nz);
@@ -522,7 +534,7 @@ thdb3ddata * thsurface::get_3d() {
         nx = - pvx->x + cvx->x;
         ny = - pvx->y + cvx->y;
         nz = - pvx->z + cvx->z;
-        nl = hypot(nz, hypot(nx, ny));
+        nl = std::hypot(nz, std::hypot(nx, ny));
         nx /= nl; ny /= nl; nz /= nl;
         nt = ny; ny = nz; nz = -nt;
         cvx->insert_normal(nx, ny, nz);
@@ -547,5 +559,15 @@ thdb3ddata * thsurface::get_3d() {
 }
 
 
-
+void thsurface::convert_all_cs() {
+	if (!thisnan(this->pict_x1)) {
+		this->convert_cs(this->pict_cs, this->pict_x1, this->pict_y1, this->pict_x1, this->pict_y1);
+	}
+	if (!thisnan(this->pict_x2)) {
+		this->convert_cs(this->pict_cs, this->pict_x2, this->pict_y2, this->pict_x2, this->pict_y2);
+	}
+	if (!thisnan(this->grid_ox)) {
+		this->convert_cs(this->grid_cs, this->grid_ox, this->grid_oy, this->grid_ox, this->grid_oy);
+	}
+}
 
