@@ -34,12 +34,21 @@
 #include "thdatabase.h"
 #include "thinfnan.h"
 #include "thinit.h"
+#include "thconfig.h"
 #include <iostream>
 #include <fstream>
 #include "thsurvey.h"
+#include "thcs.h"
 #include "thlogfile.h"
 #include "extern/img.h"
 #include <math.h>
+#include <string>
+#include <fstream>
+
+#ifdef THMSVC
+#include <direct.h>
+#define getcwd _getcwd
+#endif
 
 #define THPI 3.1415926535898
 
@@ -175,17 +184,33 @@ void thsvxctrl::write_survey_leg(thdataleg * legp)
     }
   }
 
+  double leggridmc;
+  leggridmc = 0.0;
+  if (legp->gridcs != TTCS_LOCAL) {
+    if (legp->gridcs != this->lastleggridmccs) {
+      if (legp->gridcs == thcfg.outcs) {
+        this->lastleggridmc = this->meridian_convergence;
+      } else {
+        this->lastleggridmc = thcfg.get_cs_convergence(legp->gridcs);
+      }
+      this->lastleggridmccs = legp->gridcs;
+    }
+    leggridmc = this->lastleggridmc;
+  }
+
+
   // recalculate declination if necessary
   double legp_bearing = legp->bearing;
   double legp_dx = legp->dx;
   double legp_dy = legp->dy;
-  if (!thisnan(legp->declination)) {
+  double legp_declin = (this->meridian_convergence - leggridmc + (!thisnan(legp->declination) ? legp->declination : legp->implicit_declination));
+  if (legp_declin != 0.0) {
     switch (legp->data_type) {
       case TT_DATATYPE_NORMAL:
       case TT_DATATYPE_DIVING:
       case TT_DATATYPE_CYLPOLAR:
         if (!thisnan(legp_bearing)) {
-          legp_bearing += legp->declination;
+          legp_bearing += legp_declin;
           if (legp_bearing >= 360.0)
             legp_bearing -= 360.0;
           if (legp_bearing < 0.0)
@@ -193,13 +218,25 @@ void thsvxctrl::write_survey_leg(thdataleg * legp)
         }
         break;
       case TT_DATATYPE_CARTESIAN:
-        double cosdecl = cos(legp->declination/180*THPI), 
-          sindecl = sin(legp->declination/180*THPI);
+        double cosdecl = cos(legp_declin/180*THPI), 
+          sindecl = sin(legp_declin/180*THPI);
         legp_dx = (cosdecl * legp->dx) + (sindecl * legp->dy);
         legp_dy = (cosdecl * legp->dy) - (sindecl * legp->dx);
         break;
     }
   }
+  
+  if (legp->flags != this->pdl.flags) {
+    fprintf(this->svxf,"*flags");
+    if ((legp->flags & TT_LEGFLAG_DUPLICATE) == TT_LEGFLAG_NONE) fprintf(this->svxf," not");
+    fprintf(this->svxf," duplicate");
+    if ((legp->flags & TT_LEGFLAG_SURFACE) == TT_LEGFLAG_NONE) fprintf(this->svxf," not");
+    fprintf(this->svxf," surface");
+    if ((legp->flags & TT_LEGFLAG_SPLAY) == TT_LEGFLAG_NONE) fprintf(this->svxf," not");
+    fprintf(this->svxf," splay");
+    fprintf(this->svxf,"\n");
+    this->svxf_ln++;
+  }
 
   // now print the data
   // first stations
@@ -284,7 +321,12 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
   const char * svxfn = thtmp.get_file_name("data.svx");
   this->svxf = fopen(svxfn,"w");
   if (svxf == NULL)
-    ththrow(("can't open survex file for output -- %s", svxfn))
+    ththrow("can't open survex file for output -- {}", svxfn);
+
+  this->meridian_convergence = thcfg.get_outcs_convergence();
+  this->lastleggridmccs = TTCS_LOCAL;
+  this->lastleggridmc = 0.0;
+
 
   fprintf(this->svxf,"*units declination clino compass degrees\n");
   fprintf(this->svxf,"*units tape depth counter northing easting altitude metres\n");
@@ -299,12 +341,14 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
   //thdatass_list::iterator ssi;
   thdataequate_list::iterator eqi;
   thdata * dp;
+  //thdataobject * cx1, * cx2;
+  //thdb1ds * tmps;
   
   while (obi != dbp->object_list.end()) {
   
     if ((*obi)->get_class_id() == TT_DATA_CMD) {
       
-      dp = (thdata *)(*obi);
+      dp = (thdata *)(*obi).get();
       
       // scan data shots
       lei = dp->leg_list.begin();
@@ -317,7 +361,12 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
       // scan data fixes
       fii = dp->fix_list.begin();
       while(fii != dp->fix_list.end()) {
-        this->write_survey_fix(&(*fii));
+        //tmps = &(dbp->db1d.station_vec[fii->station.id - 1]);
+        //cx1 = tmps->fixcontext;
+        //cx2 = fii->srcf.context;
+        //if (((cx1 == NULL) && (cx2 == NULL)) || ((cx1 != NULL) && (cx2 != NULL) && (cx1->id == cx2->id))) {
+          this->write_survey_fix(&(*fii));
+        //}
         fii++;
       }
   
@@ -349,7 +398,7 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
   // run survex
   thbuffer svxcom, wdir;
   wdir.guarantee(1024);
-  getcwd(wdir.get_buffer(),1024);
+  thassert(getcwd(wdir.get_buffer(),1024) != NULL);
   int retcode;
   svxcom = "\"";
   svxcom += thini.get_path_cavern();
@@ -368,7 +417,9 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
   this->transcript_log_file(dbp, thtmp.get_file_name("data.log"));
 
   if (retcode != EXIT_SUCCESS)
-    ththrow(("cavern exit code -- %d", retcode))
+    ththrow("cavern exit code -- {}", retcode);
+  else
+    this->load_err_file(dbp, thtmp.get_file_name("data.err"));
 
 
   // read coordinates from 3D file
@@ -378,7 +429,7 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
   thdb1ds * stp;
   img* pimg = img_open(thtmp.get_file_name("data.3d"));
   if (pimg == NULL)
-    ththrow(("can't open cavern output"))    
+    ththrow("can't open cavern output");
   do {
     result = img_read_item(pimg, &imgpt);
     switch (result) {
@@ -395,7 +446,7 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
         break;
       case img_BAD:
         img_close(pimg);
-        ththrow(("error reading cavern output"))
+        ththrow("error reading cavern output");
         break;
     }
   } while (result != img_STOP);
@@ -415,7 +466,7 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
 //   retcode = system(svxcom.get_buffer());
 // 
 //   if (retcode != EXIT_SUCCESS)
-//     ththrow(("3dtopos exit code -- %d", retcode))
+//     ththrow("3dtopos exit code -- %d", retcode);
 // 
 // // Let's copy results and log-file to working directory
 // // #ifdef THWIN32
@@ -433,7 +484,7 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
 // //   retcode = system(svxcom.get_buffer());
 // // 
 // //   if (retcode != EXIT_SUCCESS)
-// //     ththrow(("cp exit code -- %d", retcode))
+// //     ththrow("cp exit code -- %d", retcode);
 //   
 //   
 // #ifdef THDEBUG
@@ -451,7 +502,7 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
 //     * p[4], * cps = lnbuff;
 //   posf.open(thtmp.get_file_name("data.pos"));
 //   if (!posf.is_open())
-//     ththrow(("can't open survex output"))
+//     ththrow("can't open survex output");
 //   while (!posf.eof()) {
 //     posf.getline(lnbuff, lnsize);
 //     clns = strlen(lnbuff);
@@ -505,7 +556,7 @@ void thsvxctrl::process_survey_data(class thdatabase * dbp)
 //  FILE * ptsf;
 //  ptsf = fopen("data.pts","w");  
 //  if (ptsf == NULL)
-//    ththrow(("can't open data.pts for output"))
+//    ththrow("can't open data.pts for output");
 //  unsigned int sid, lsid;
 //  lsid = dbp->db1d.station_vec.size();
 //  for (sid = 0; sid < lsid; sid++) {
@@ -529,13 +580,13 @@ void thsvxctrl::transcript_log_file(class thdatabase * dbp, const char * lfnm)
   thlog.printf("\n####################### cavern log file ########################\n");
   std::ifstream clf(lfnm);
   if (!(clf.is_open()))
-    ththrow(("can't open cavern log file for input"))
+    ththrow("can't open cavern log file for input");
   // let's read line by line and print to log file
   size_t chidx, nchs;
   char * chch;
   bool onnum, ondig, fonline;
   long csn;
-  unsigned int lsid;
+  size_t lsid;
   lsid = dbp->db1d.station_vec.size();
   while (!(clf.eof())) {
     lnum++;
@@ -551,6 +602,7 @@ void thsvxctrl::transcript_log_file(class thdatabase * dbp, const char * lfnm)
     char * start_ch = NULL; //, * test_ch;
     int num_type;
     thsvxctrl_src_maptype::iterator srcmi;
+    const auto prev_char_is = [&lnbuff, &start_ch](const char c) { return (start_ch - 1) >= lnbuff && start_ch[-1] == c; };
 //    if (*lnbuff == 13) lnbuff++;
 //    if (strncmp(lnbuff,"There were",10) == 0)
 //      chidx = 2049;
@@ -586,7 +638,7 @@ void thsvxctrl::transcript_log_file(class thdatabase * dbp, const char * lfnm)
         else {
           num_type = THSVXLOGNUM_STATION;
           if ((chidx > 0) && (chidx <= nchs)) {
-            if ((start_ch[-1] == ':') && (*chch == ':'))
+            if (prev_char_is(':') && (*chch == ':'))
               num_type = THSVXLOGNUM_LINE;
           }
           if (num_type == THSVXLOGNUM_STATION) {
@@ -594,7 +646,7 @@ void thsvxctrl::transcript_log_file(class thdatabase * dbp, const char * lfnm)
               if ((*chch == '.') || (*chch == ',') || (*chch == ')') || (*chch == 'm')
                 || (*chch == '-') || (*chch == '/'))
                 num_type = THSVXLOGNUM_NONE;
-              if ((start_ch[-1] == '.') || (start_ch[-1] == '-'))
+              if (prev_char_is('.') || prev_char_is('-'))
                 num_type = THSVXLOGNUM_NONE;
             }
           }
@@ -651,3 +703,106 @@ void thsvxctrl::transcript_log_file(class thdatabase * dbp, const char * lfnm)
   thlog.printf("#################### end of cavern log file ####################\n");
   delete [] lnbuff;
 }
+
+void thsvxctrl::load_err_file(class thdatabase * dbp, const char * lfnm) {
+
+	std::string line;
+	std::ifstream infile(lfnm);
+	thmbuffer b;
+	unsigned long sf, st, prev_st, nst;
+	long i;
+
+	while(std::getline(infile, line)) {
+
+//		thprintf("SCANNING: %s\n", line.c_str());
+		thsplit_args(&b, line.c_str());
+		thdb1d_traverse t, * ct;
+		thdb1d_loop_leg l;
+		thdb1d_loop_leg * cl;
+		thdb1d_tree_node * cn;
+		thdb1d_tree_arrow * ca;
+		bool has_legs;
+		has_legs = false;
+		nst = 0;
+		cl = NULL;
+		prev_st = 0;
+
+		i = 0;
+		if (b.get_size() > 2) {
+			i = 0;
+			while (i < b.get_size()) {
+//				thprintf("%s\n", b.get_buffer()[i]);
+			  st = 0;
+			  sscanf(b.get_buffer()[i], "%lu", &st);
+				if (st > 0) {
+					nst++;
+					if (nst == 1) {
+						sf = st;
+//						thprintf("\nFROM: %lu\n", sf);
+						t.first_leg = NULL;
+						t.last_leg = NULL;
+						t.from = & (dbp->db1d.station_vec[sf-1]);
+						t.nlegs = 0;
+						prev_st = sf;
+						cl = NULL;
+					} else {
+						t.to = & (dbp->db1d.station_vec[st-1]);
+						if (strcmp(b.get_buffer()[i-1],"-") == 0) {
+							has_legs = true;
+							l.prev_leg = cl;
+							l.next_leg = NULL;
+							l.leg = NULL;
+							l.reverse = false; //TODO
+							// find appropriate leg
+							cn = &(dbp->db1d.tree_nodes[dbp->db1d.station_vec[prev_st-1].uid - 1]);
+							for(ca = cn->first_arrow; ca != NULL; ca = ca->next_arrow) {
+								if (ca->end_node->uid == dbp->db1d.station_vec[st-1].uid) {
+									l.leg = ca->leg->leg;
+									l.reverse = ca->leg->reverse;
+									break;
+								}
+							}
+							if (l.leg != NULL) {
+								cl = &(*dbp->db1d.loop_leg_list.insert(dbp->db1d.loop_leg_list.end(), l));
+								if (t.nlegs == 0) {
+									t.first_leg = cl;
+								}
+								if (cl->prev_leg != NULL) {
+									cl->prev_leg->next_leg = cl;
+								}
+								t.last_leg = cl;
+								t.nlegs++;
+							}
+							else {
+//								thprintf("LEG not found!!!\n");
+							}
+//							thprintf("TO: %s %lu\n", b.get_buffer()[i], st);
+						}
+						else if (strcmp(b.get_buffer()[i-1],"=") == 0) {
+//							thprintf("EQ: %s %lu\n", b.get_buffer()[i], st);
+						}
+						prev_st = st;
+					}
+				}
+				i++;
+			}
+		}
+		if (has_legs) {
+			t.id = dbp->db1d.traverse_list.size() + 1;
+			// TODO: read original length
+			std::getline(infile, line);
+			// read E
+			std::getline(infile, line);
+			sscanf(line.c_str(), "%lf", &t.E);
+			// TODO: read H, V
+			std::getline(infile, line);
+//			thprintf("E: %.2lf\n", t.E);
+			ct = &(*dbp->db1d.traverse_list.insert(dbp->db1d.traverse_list.end(), t));
+			for(cl = ct->first_leg; cl != NULL; cl = cl->next_leg) {
+				cl->leg->traverse = ct;
+			}
+		}
+	}
+
+
+}
